## procs
proc writeTop {file segResList} {    
    ## write topology file

    ## base: 158.096 + 8 H = 166	    
    ## without H: {ADE 182.1223907470703} {CYT 158.0968017578125} {THY 172.10020446777344} {GUA 198.12179565429688}
    ## num H on base: {ADE 8} {CYT 8} {THY 9} {GUA 8}
    ## base mass H: {ADE 190.1} {CYT 166.1} {THY 181.1} {GUA 206.1}
    ## base: 158.096 + 8 H = 166	    
    set ch [open $file w]
    puts $ch [format {MASS %-2i   %-4s   160.1     ! nucleotide backbone} 1 P]
    puts $ch [format {MASS %-2i   %-4s   181.1     ! nucleotide base} 2 B]
    puts $ch "" 
    puts $ch "DEFA FIRS none LAST none"

    puts $ch [format {RESI %-4s         0.00} N]
    puts $ch [format {ATOM P    %-4s   0.00} P]
    puts $ch [format {ATOM B    %-4s   0.00} B]
    # puts $ch {BOND P B}
    puts $ch ""

    puts $ch {PRES BOND         0.00}
    puts $ch {BOND P B}
    puts $ch {PRES LINK         0.00}
    puts $ch {BOND 1P 2P}
    puts $ch {BOND 1B 2P}
    puts $ch "end" 
    close $ch
}
proc writeParamAndExtraBonds {paramFile extraBondsFile segResList} {
    ## write parameter file
    set pCh [open $paramFile w]
    set eCh [open $extraBondsFile w]

    set bondFormat "bond %i %i %s"
    set angleFormat "angle %i %i %i %s"
    set dihedralFormat "dihedral %i %i %i %i %s"
    set excludeFormat "exclude %i %i"

    puts $pCh "BONDS
!V(bond) = Kb(b - b0)**2
!
!Kb: kcal/mole/A**2
!b0: A
!
!atom type Kb          b0"
    # CN8  NN6    200.000     1.480   ! methylammonium
    puts $pCh [format {%-4s%-4s     0.001       6.00    3} P B]
    puts $pCh [format {%-4s%-4s     0.001       6.00    3} P P]
    puts $pCh "ANGLES"
    puts $pCh "DIHEDRALS"
    puts $pCh "IMPROPER"
    puts $pCh "NONBONDED  NBXMOD 5  ATOM CDIEL FSHIFT VATOM VDISTANCE VFSWITCH -
     CUTNB 14.0  CTOFNB 12.0  CTONNB 10.0  EPS 1.0  E14FAC 1.0  WMIN 1.5"

    ## add a blank LJ entry for each bead
    puts $pCh "!atom  ignored    epsilon      Rmin/2   ignored   eps,1-4       Rmin/2,1-4"
    puts $pCh [format {%-4s     0.0       -0.00    3} P]
    puts $pCh [format {%-4s     0.0       -0.00    3} B]
    
    ## bonded and non-bonded interactions are specified below
    puts $pCh "NBTABLE"
    puts $pCh "P P NBPP"
    puts $pCh "P B NBPB"
    puts $pCh "B B NBBB"

    ## intra-chain interactions
    set segI 0
    foreach {seg resList} $segResList {
	## intra-residue interactions
	set i $segI
	foreach res1 $resList {
	    puts $eCh [format $bondFormat [expr 2*$i] [expr 2*$i+1] BPB];# BPB is for bonded P1-B1
	    puts $eCh [format $excludeFormat [expr 2*$i] [expr 2*$i+1]]
	    incr i
	}
	
	## interactions between 2 consecutive nts
	set i $segI	
	foreach res1 [lrange $resList 0 end-1] {
	    set j [expr $i+1]
	    puts $eCh [format $bondFormat [expr 2*$i] [expr 2*$j] BPP]
	    puts $eCh [format $bondFormat [expr 2*$i+1] [expr 2*$j] BBP];# BBP is for bonded B1-P2
	    puts $eCh [format $angleFormat [expr 2*$i] [expr 2*$j] [expr 2*$j+1] p1p2b2]
	    puts $eCh [format $angleFormat [expr 2*$i+1] [expr 2*$j] [expr 2*$j+1] b1p2b2]
	    # puts $eCh [format $dihedralFormat  [expr 2*$i] [expr 2*$i+1]  [expr 2*$j] [expr 2*$j+1] p1b1p2b2]
	    puts $eCh [format $dihedralFormat  [expr 2*$i+1] [expr 2*$i]  [expr 2*$j] [expr 2*$j+1] b1p1p2b2]

	    puts $eCh [format $excludeFormat [expr 2*$i]   [expr 2*$j]]
	    puts $eCh [format $excludeFormat [expr 2*$i+1] [expr 2*$j]]
	    puts $eCh [format $excludeFormat [expr 2*$i]   [expr 2*$j+1]]
	    puts $eCh [format $excludeFormat [expr 2*$i+1] [expr 2*$j+1]]
	    
	    incr i
	}

	## interactions between 3 consecutive nts
	set i $segI	
	foreach res1 [lrange $resList 0 end-2] {
	    set j [expr $i+1]
	    set k [expr $i+2]
	    ## angle between consecutive phosphates
	    puts $eCh [format $angleFormat [expr 2*$i] [expr 2*$j] [expr 2*$k] p1p2p3]
	    puts $eCh [format $angleFormat [expr 2*$i+1] [expr 2*$j] [expr 2*$k] b1p2p3] ;# possibly need to comment this one out!!! (seems okay)
	    puts $eCh [format $dihedralFormat  [expr 2*$i+1] [expr 2*$j]  [expr 2*$k] [expr 2*$k+1] b1p2p3b3]

	    puts $eCh [format $excludeFormat [expr 2*$i]   [expr 2*$k]]
	    puts $eCh [format $excludeFormat [expr 2*$i+1] [expr 2*$k]]
	    incr i
	}

	## interactions between 4 consecutive nts
	set i $segI	
	foreach res1 [lrange $resList 0 end-3] {
	    set j [expr $i+1]
	    set k [expr $i+2]
	    set l [expr $i+3]
	    ## dihedral between consecutive phosphates
	    puts $eCh [format $dihedralFormat  [expr 2*$i] [expr 2*$j]  [expr 2*$k] [expr 2*$l] p0p1p2p3]
	    incr i
	}
	incr segI [llength $resList]
    }
    close $pCh
    close $eCh
}

proc addBead {atomText seg res name} {
    variable RID
    set atoms [atomselect $RID $atomText]
    coord $seg $res $name [measure center $atoms weight mass]
}

proc writePdbPsf {out segResList top} {
    package require psfgen
    resetpsf
    
    topology $top
    set i 0
    foreach {seg resList} $segResList {
	puts "Working on seg $seg"
	set j $i
	segment $seg {
	    foreach res $resList {
		puts "residue $j N"
		residue $j N
		incr j
	    }
	}
	
	foreach res1 [join "0 [lrange $resList 0 end-1]"] res2 $resList {
	    puts "Adding coordinates for r$res1+r$res2 --> $i"
	    addBead "segname $seg and ((resid $res1 and name C3' O3') or (resid $res2 and name O5T' O5' P O1P O2P C5'))" $seg $i P
	    addBead "segname $seg and resid $res2 and noh and not name C3' O3' O5' O5T' P O1P O2P C5'" $seg $i B
	    incr i
	}
    }

    writepsf $out.psf
    writepdb $out.pdb
    
    ## apply bonding patches
    set i 0
    foreach {seg resList} $segResList {
	foreach res [lrange $resList 0 end-1] {
	    set j [expr $i+1]
	    puts "patching with: patch LINK $seg:$i $seg:$j"
	    # patch BOND $seg:$i
	    patch LINK $seg:$i $seg:$j
	    incr i 
	}
	# patch BOND $seg:$i
	incr i ;# for extra nt at end
    }

    writepsf $out.bonded.psf
}
