## this script builds some DNA

### arguments
lassign $argv outPsf
regexp {(.*).psf} $outPsf --> out
regexp {.*-([0-9]+)} $out --> dnaLength


### path
proc randGauss {{stdDev 1}} {
    expr { $stdDev * sqrt( -2*log(rand())) * cos( 2*3.14159*rand() ) }
}
proc randomUnitVector {} {
    set v [list [randGauss] [randGauss] [randGauss]]
    vecnorm $v
}

#- do a running average over 3-4 random directions, that way you get a roughly correct 
set ntLength 6.2 ;# P-P length
set windowSize 4
set dirWindow ""
while { [llength $dirWindow] < $windowSize } { 
    lappend dirWindow [randomUnitVector]
}


### some procs
source 1-build.procs.tcl
set pos {0 0 0}
proc updateDir {} {
    variable windowSize
    variable dirWindow
    variable ntLength
    variable dir

    set dir "0 0 0"
    foreach d $dirWindow {
	set dir [vecadd $dir $d]
    }
    set dir [vecnorm $dir]
    set dir [vecscale $ntLength $dir]
    
    set dirWindow [lrange $dirWindow 1 end]
    lappend dirWindow [randomUnitVector]
    puts $dir
}
updateDir
proc addBead {atomText seg res name} {
    variable ID
    variable M
    variable pos
    variable dir
    puts $atomText
    
    if { $name == "B" } {
	set v [vecadd $pos [vecscale 6 [vecnorm [vecadd [vecnorm $dir] [vecscale 1.5 [vecnorm [veccross $dir [randomUnitVector]]]]]]]]
    } else {
	# set pos [vecadd $pos [vecscale 6.2 $dir]]
	set pos [vecadd $pos $dir]
	set v $pos
	## update dir
	updateDir
    }
    coord $seg $res $name $v
}
proc writePdbPsf {out segResList top} {
    package require psfgen
    resetpsf
    
    topology $top
    set i 0
    foreach {seg resList} $segResList {
	puts "Working on seg $seg"
	set j $i
	segment $seg {
	    foreach res $resList {
		puts "residue $j N"
		residue $j N
		incr j
	    }
	}
	foreach res1 [join "0 [lrange $resList 0 end-1]"] res2 $resList {
	    puts "Adding coordinates for r$res1+r$res2 --> $i"
	    addBead "segname $seg and resid $res2 and name P" $seg $i P
	    addBead "segname $seg and resid $res2 and name B" $seg $i B
	    incr i
	}
    }

    writepsf $out.psf
    writepdb $out.pdb

    ## connect nucleotides
    set i 0
    foreach {seg resList} $segResList {
	foreach res [lrange $resList 0 end-1] {
	    set j [expr $i+1]
	    puts "patching with: patch LINK $seg:$i $seg:$j"
	    patch BOND $seg:$i
	    patch LINK $seg:$i $seg:$j
	    incr i 
	}
	patch BOND $seg:$i
	incr i ;# for extra nt at end
    }
    writepsf $out.bonded.psf
}

### build segResList
set segResList ""
set seg ADNA
set fullResList ""
set numRes 0
set resList ""
for {set i 0} {$i < $dnaLength} {incr i} {
    lappend resList $i
}
set segResList "$seg {$resList}"

puts "writing top"
writeTop $out.top $segResList
puts "writing params"
writeParamAndExtraBonds $out.par $out.extraBonds.txt $segResList
puts "writing psf/pdb"
writePdbPsf $out $segResList $out.top
