proc getLastSimulationNumber2 {prefix {outDir output} {ext .coor}} {
    set nums 0

    set prefix $outDir/$prefix
    print "getLastSimulationNumber2: ${prefix}*$ext*"
    if { [catch { glob ${prefix}*$ext* }] } {
	return 0
    }
    set files [glob ${prefix}*$ext]

    print "getLastSimulationNumber2: files = $files"
    set reNum {(-([0-9]+))+}
    set reExt {(.restart)?}
    set reSuff {(.old)?}
    set regExp "${prefix}${reNum}(${reExt}${ext}${reSuff})"
    #print "getLastSimulationNumber2: regepx = $regExp"
    foreach file $files {
	set num 0                             ;# prevents error if regexp does not match
        #regexp $regExp $file trash num
	set match [regexp -inline $regExp $file]
	set num [lindex $match end-3]
	print "getLastSimulationNumber2: match = $match ; num = $num"
	if { [regexp {[-]?[0-9]+} $num] } {
	    lappend nums $num
	}
    }
    print "getLastSimulationNumber2: [lsort -decreasing -integer $nums]"
    return [lindex [lsort -decreasing -integer $nums] 0]
}
proc getLastSimulationNumber {prefix {outDir output} {ext .coor}} {
    set nums 0

    set prefix $outDir/$prefix
    print "getLastSimulationNumber: ${prefix}*$ext*"
    if { [catch { glob ${prefix}*$ext* }] } {
	return 0
    }
    set files [glob ${prefix}*$ext]

    print "getLastSimulationNumber: files = $files"
    set reNum {([0-9]+)}
    set reExt {(.restart)?}
    set reSuff {(.old)?}
    set regExp ${prefix}${reNum}${reExt}${ext}${reSuff}
    foreach file $files {
	set num 0                             ;# prevents error if regexp does not match
        regexp $regExp $file trash num
        lappend nums $num
    }
    return [lindex [lsort -decreasing -integer $nums] 0]
}
proc fileExistsAndIsNonempty {file} {
    set file [followLink $file]
    print "fileExistsAndIsNonempty $file"
    print [fileExists $file]
    
    if { [fileExists $file] } {
	if { [file size $file] != 0 } {
	    return 1
	}
    }
    return 0
}
proc fileExists {file} {
    ## built in tcl proc "file exists" has given me false positives on Jaguar, so I'm going to use something more robust
    if { [catch "file size $file"] } { return 0 }
    return 1
}
proc followLink {file} {
    if { [catch "file link $file"] } {
	return $file
    } else {
	return [file dirname $file]/[file link $file]
    }
}
proc fileSize {file} {
    set file [followLink $file]
    if { ! [fileExists $file] } { return 0 }
    return [file size $file]
}
proc filesAreSameSize {file1 file2} {
    set file1 [followLink $file1]
    set file2 [followLink $file2]
    if { [fileExists $file1] && [fileExists $file2] } {
	if { [file size $file1] == [file size $file2] } {
	    return 1
	} else { puts "$file1 is not the same size as $file2"; return 0 }
    } else { return 1 }
}
proc getColvarsRestartFile {prefix {outDir output}} {
    set lastRun $outDir/$prefix
    set returnList ""
    set ext colvars.state
    set cmd {
	if { [fileExistsAndIsNonempty $lastRun.$ext] && [filesAreSameSize $lastRun.$ext $lastRun.$ext.old] } {
	    lappend returnList $lastRun.$ext
	} elseif { [fileExistsAndIsNonempty $lastRun.$ext.old] } { ## xsc.old
	    lappend returnList $lastRun.$ext.old
	} else { error "getColvarsRestartFile: Error: neither $lastRun.$ext nor $lastRun.$ext.old exists and is non-empty." }
    }
    ## evaluate
    if { [catch $cmd] } {
	print "getColvarsRestartFile: Restart files without .restart prefix do not exist."
	set returnList ""
	set lastRun $lastRun.restart
	eval $cmd
    }
    return $returnList
}    
    
proc getRestartFiles3 {prefix {outDir output}} {
    ## returns ".xsc .coor .vel"
    set lastRun $outDir/$prefix
    set returnList ""
    
    set useOldIfPossible 0
    set cmd {
	foreach ext {vel coor xsc} {
	    set restartRank [expr [fileExistsAndIsNonempty $lastRun.$ext] * ([fileSize $lastRun.$ext] + 0.5)]
	    set oldRank [expr [fileExistsAndIsNonempty $lastRun.$ext.old] * ([fileSize $lastRun.$ext.old] + $useOldIfPossible)]
	
	    if { $restartRank == 0 && $oldRank == 0 } {
		 error "getRestartFiles: Error: neither $lastRun.$ext nor $lastRun.$ext.old exists and is non-empty." 
	    } elseif { $restartRank >= $oldRank } {
		lappend returnList $lastRun.$ext
		set useOldIfPossible 0
	    } else {
		lappend returnList $lastRun.$ext.old
		set useOldIfPossible 1
	    } 
	}
    }
    
    ## evaluate
    if { [catch $cmd] } {
	print "getRestartFiles: Restart files without .restart prefix do not exist."
	set returnList ""
	set lastRun $lastRun.restart
	eval $cmd
    }

    return $returnList
}
proc getRestartFiles2 {prefix {outDir output}} {
    # returns ".xsc .coor .vel"
    set lastRun $outDir/$prefix
    set returnList ""
    
    set useOldIfPossible 0
    set cmd {
	set ext xsc
	if { [fileExistsAndIsNonempty $lastRun.$ext] } {
	    lappend returnList $lastRun.$ext
	} elseif { [fileExistsAndIsNonempty $lastRun.$ext.old] } {
	    lappend returnList $lastRun.$ext.old
	    set useOldIfPossible 1
	} else { error "getRestartFiles: Error: neither $lastRun.$ext nor $lastRun.$ext.old exists and is non-empty." }

	foreach ext {vel coor} {
	    set restartRank [expr [fileExistsAndIsNonempty $lastRun.$ext] * ([fileSize $lastRun.$ext] + 0.5)]
	    set oldRank [expr [fileExistsAndIsNonempty $lastRun.$ext.old] * ([fileSize $lastRun.$ext.old] + $useOldIfPossible)]
	
	    if { $restartRank == 0 && $oldRank == 0 } {
		 error "getRestartFiles: Error: neither $lastRun.$ext nor $lastRun.$ext.old exists and is non-empty." 
	    } elseif { $restartRank >= $oldRank } {
		lappend returnList $lastRun.$ext
		set useOldIfPossible 0
	    } else {
		lappend returnList $lastRun.$ext.old
		set useOldIfPossible 1
	    } 
	}
    }
    
    ## evaluate
    if { [catch $cmd] } {
	print "getRestartFiles: Restart files without .restart prefix do not exist."
	set returnList ""
	set lastRun $lastRun.restart
	eval $cmd
    }

    return $returnList
}
proc getRestartFiles.BAK {prefix {outDir output}} {
    # returns ".xsc .coor .vel"
    set lastRun $outDir/$prefix
    set returnList ""
    
    set cmd {
	set ext xsc
	if { [fileExistsAndIsNonempty $lastRun.$ext] } {
	    lappend returnList $lastRun.$ext
	} elseif { [fileExistsAndIsNonempty $lastRun.$ext.old] } {
	    lappend returnList $lastRun.$ext.old
	} else { error "getRestartFiles: Error: neither $lastRun.$ext nor $lastRun.$ext.old exists and is non-empty." }

	foreach ext {coor vel} {
	    # print [fileExistsAndIsNonempty $lastRun.$ext]
	    # print [filesAreSameSize $lastRun.$ext $lastRun.$ext.old]
	    if { [fileExistsAndIsNonempty $lastRun.$ext] && ([fileSize $lastRun.$ext] >= [fileSize $lastRun.$ext.old]) } {
		# [filesAreSameSize $lastRun.$ext $lastRun.$ext.old]
		lappend returnList $lastRun.$ext
	    } elseif { [fileExistsAndIsNonempty $lastRun.$ext.old] } {
		lappend returnList $lastRun.$ext.old
	    } else { error "getRestartFiles: Error: neither $lastRun.$ext nor $lastRun.$ext.old exists and is non-empty." }
	}
	
    }
    
    ## evaluate
    if { [catch $cmd] } {
	print "getRestartFiles: Restart files without .restart prefix do not exist."
	set returnList ""
	set lastRun $lastRun.restart
	eval $cmd
    }

    return $returnList
}

proc getRestartPrefixAndSuffix {prefix {outDir output}} {
    print "getRestartPrefixAndSuffix $prefix $outDir"
    set LastRun $outDir/$prefix
    set suffix ""

    ## check whether $prefix.$ext exists and coor and vel are same size 
    set filesExist 1
    foreach ext {coor vel xsc} { ## check existence
	if { ! [fileExists $LastRun.$ext] } {
	    set filesExist 0
	}
    }
    if { $filesExist } {
	if { [file size $LastRun.coor] != [file size $LastRun.vel] } { ## check size of coor and vel
	    set filesExist 0
	}
	if { ([file size $LastRun.coor] * [file size $LastRun.vel] == 0) } { ## make sure files are not empty
	    set filesExist 0
	}
    }
    if { $filesExist } {
	return "$prefix $suffix"
    }
    
    ## check restart files
    set LastRun $LastRun.restart
    set prefix $prefix.restart
    set filesExist 1
    foreach ext {coor vel xsc} {
	if { ! [fileExists $LastRun.$ext] } {
	    set filesExist 0
	}
    }
    if { $filesExist } {
	if { ([file size $LastRun.coor] != [file size $LastRun.vel]) } { ## make sure files are the same size
	    set filesExist 0
	}
	if { ([file size $LastRun.coor] * [file size $LastRun.vel] == 0) } { ## make sure files are not empty
	    set filesExist 0
	}
    }
    if { $filesExist == 0 } { ## if this doesn't work, nothing will... go ahead and fix things by hand if namd fails
	set suffix .old
    }
    return "$prefix $suffix"
}
proc getLastTimestepFromXsc {xsc} {
    print "getLastTimestep: pwd = [pwd]"
    print "getLastTimestep: xsc = $xsc"

    if { [fileExists $xsc ] } {
	print "getLastTimestep: $xsc exists"
    } else {
	error "getLastTimestep: $xsc does not exist!!!"
    }

    set line ""
    set lastLine ""

    puts "trying to open \"$xsc\""
    set ch [open $xsc]
    print "getLastTimestep: file open"
    while { [gets $ch line] >= 0 } {
	print "getLastTimestep: line = $line"
	set lastLine $line
    }
    close $ch

    set line $lastLine
    print "getLastTimestep: lastline = $line"
    return [lindex $line 0]
}
    
proc getLastTimestep {lastRun {suffix ""}} {
    # lindex [concat [exec tail -n 1 ${LastRun}.xsc]] 0
    print "getLastTimestep: pwd = [pwd]"
    print "getLastTimestep: lastRun = $lastRun"
    print "getLastTimestep: suffix = $suffix"
    # set file $lastRun.xsc$suffix
    if { [fileExists $file ] } {
	print "getLastTimestep: $file exists"
    } else {
	print "getLastTimestep: $file does not exist!!!"
    }
    puts "trying to open \"$file\""
    set ch [open $file]
    print "getLastTimestep: file open"
    set line ""
    set lastLine ""
    while { [gets $ch line] >= 0 } {
	print "getLastTimestep: line = $line"
	set lastLine $line
    }
    if { ! [info exists line] } {
	print "getLastTimestep: variable line does not exist: checking with .old appended to suffix"
	return [getLastTimestep $lastRun $suffix.old]
    } elseif { [string equal $lastLine ""] } {
	print "getLastTimestep: lastLine is empty: checking with .old appended to suffix"
	return [getLastTimestep $lastRun $suffix.old]
    }
    set line $lastLine
    print "getLastTimestep: lastline = $line"
    
    # print [subst -nocommands {[lindex [concat [exec tail -n 1 $file]] 0]}]
    #lindex [concat [exec tail -n 1 $file]] 0
    return [lindex $line 0]
}
